@@ -1,6 +1,5 @@
-#!/usr/bin/env python2
-# vim:fileencoding=utf-8
-from __future__ import absolute_import, division, print_function, unicode_literals
+#!/usr/bin/env python
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2016, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -75,7 +74,7 @@ class CascadeTest(BaseTest):
         get_rules({'x/one.css':'@media screen { body { color: red; } }'})
         get_rules({'x/one.css':'@media xyz { body { color: red; } }'}, l=0)
         c = get_rules({'x/one.css':'@import "../two.css";', 'two.css':'@import "x/one.css"; body { color: red; }'})[1]
-        self.assertIn('Recursive import', c.log_stream.getvalue().decode('utf-8'))
+        self.assertIn('Recursive import', c.log_stream.getvalue())
 
     def test_resolve_styles(self):
 
@@ -83,7 +82,7 @@ class CascadeTest(BaseTest):
             elem = next(select(selector))
             ans = resolve_property(elem, name)
             if val is None:
-                val = type('')(DEFAULTS[name])
+                val = str(DEFAULTS[name])
             self.assertEqual(val, ans.cssText)
 
         def test_pseudo_property(select, resolve_pseudo_property, selector, prop, name, val=None, abort_on_missing=False):
@@ -94,11 +93,11 @@ class CascadeTest(BaseTest):
                     self.assertTrue(ans is None)
                     return
             if val is None:
-                val = type('')(DEFAULTS[name])
+                val = str(DEFAULTS[name])
             self.assertEqual(val, ans.cssText)
 
         def get_maps(html, styles=None, pseudo=False):
-            html = '<html><head><link href="styles.css"></head><body>{}</body></html>'.format(html)
+            html = f'<html><head><link href="styles.css"></head><body>{html}</body></html>'
             c = VirtualContainer({'index.html':html, 'styles.css':styles or 'body { color: red; font-family: "Kovid Goyal", sans-serif }'})
             resolve_property, resolve_pseudo_property, select = resolve_styles(c, 'index.html')
             if pseudo:
@@ -151,7 +150,7 @@ class CascadeTest(BaseTest):
 
         def get_stats(html, *fonts):
             styles = []
-            html = '<html><head><link href="styles.css"></head><body>{}</body></html>'.format(html)
+            html = f'<html><head><link href="styles.css"></head><body>{html}</body></html>'
             files = {'index.html':html, 'X.otf':b'xxx', 'XB.otf': b'xbxb'}
             for font in fonts:
                 styles.append('@font-face {')
@@ -159,9 +158,9 @@ class CascadeTest(BaseTest):
                     if k == 'src':
                         files[v] = b'xxx'
                         v = 'url(%s)' % v
-                    styles.append('%s : %s;' % (k, v))
+                    styles.append(f'{k} : {v};')
                 styles.append('}\n')
-            html = '<html><head><link href="styles.css"></head><body>{}</body></html>'.format(html)
+            html = f'<html><head><link href="styles.css"></head><body>{html}</body></html>'
             files['styles.css'] = embeds + '\n'.join(styles)
             c = VirtualContainer(files)
             return StatsCollector(c, do_embed=True)
@@ -213,7 +212,7 @@ class CascadeTest(BaseTest):
         style = parseStyle('background-image: url(b.png); background: black url(a.png) fixed')
         for prop in style.getProperties(all=True):
             remove_property_value(prop, lambda val:'png' in val.cssText)
-        self.assertEqual('background: black fixed', style.cssText)
+        self.assertEqual('background: black fixed', style.cssText.rstrip(';'))
 
     def test_fallback_font_matching(self):
         def cf(id, weight='normal', style='normal', stretch='normal'):
