@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 # License: GPLv3 Copyright: 2010, Kovid Goyal <kovid at kovidgoyal.net>
 
 
@@ -8,27 +7,30 @@ import re
 import weakref
 from collections import defaultdict
 from contextlib import contextmanager
-
 from html5_parser import parse
 from lxml import html
 from qt.core import (
     QAction, QApplication, QBrush, QByteArray, QCheckBox, QColor, QColorDialog,
-    QDialog, QDialogButtonBox, QFont, QFontInfo, QFontMetrics, QFormLayout,
-    QHBoxLayout, QIcon, QKeySequence, QLabel, QLineEdit, QMenu, QPalette,
-    QPlainTextEdit, QPushButton, QSize, QSyntaxHighlighter, Qt, QTabWidget,
-    QTextBlockFormat, QTextCharFormat, QTextCursor, QTextEdit, QTextListFormat,
-    QToolBar, QUrl, QVBoxLayout, QWidget, pyqtSignal, pyqtSlot, QToolButton, QTextFormat
+    QDialog, QDialogButtonBox, QFont, QFontInfo, QFontMetrics, QFormLayout, QIcon,
+    QKeySequence, QLabel, QLineEdit, QMenu, QPalette, QPlainTextEdit, QPushButton,
+    QSize, QSyntaxHighlighter, Qt, QTabWidget, QTextBlockFormat, QTextCharFormat,
+    QTextCursor, QTextEdit, QTextFormat, QTextListFormat, QTimer, QToolButton, QUrl,
+    QVBoxLayout, QWidget, pyqtSignal, pyqtSlot
 )
 
 from calibre import xml_replace_entities
 from calibre.ebooks.chardet import xml_to_unicode
-from calibre.gui2 import NO_URL_FORMATTING, choose_files, error_dialog, gprefs, is_dark_theme
+from calibre.gui2 import (
+    NO_URL_FORMATTING, choose_files, error_dialog, gprefs, is_dark_theme
+)
 from calibre.gui2.book_details import css
+from calibre.gui2.flow_toolbar import create_flow_toolbar
 from calibre.gui2.widgets import LineEditECM
 from calibre.gui2.widgets2 import to_plain_text
+from calibre.utils.cleantext import clean_xml_chars
 from calibre.utils.config import tweaks
 from calibre.utils.imghdr import what
-from polyglot.builtins import filter, iteritems, itervalues, unicode_type
+from polyglot.builtins import iteritems, itervalues
 
 # Cleanup Qt markup {{{
 
@@ -101,13 +103,13 @@ def remove_heading_font_styles(tag, style):
     expected_size = (None, 'xx-large', 'x-large', 'large', None, 'small', 'x-small')[lvl]
     if style.get('font-size', 1) == expected_size:
         del style['font-size']
-    if style.get('font-weight') == '600':
+    if style.get('font-weight') in ('600', '700', 'bold'):
         del style['font-weight']
 
 
 def use_implicit_styling_for_span(span, style):
     is_italic = style.get('font-style') == 'italic'
-    is_bold = style.get('font-weight') == '600'
+    is_bold = style.get('font-weight') in ('600', '700', 'bold')
     if is_italic and not is_bold:
         del style['font-style']
         span.tag = 'em'
@@ -137,7 +139,7 @@ def use_implicit_styling_for_a(a, style_map):
 def merge_contiguous_links(root):
     all_hrefs = set(root.xpath('//a/@href'))
     for href in all_hrefs:
-        tags = root.xpath('//a[@href="{}"]'.format(href))
+        tags = root.xpath(f'//a[@href="{href}"]')
         processed = set()
 
         def insert_tag(parent, child):
@@ -204,9 +206,12 @@ def cleanup_qt_markup(root):
                     remove_zero_indents(ts)
     for style in itervalues(style_map):
         filter_qt_styles(style)
+        fw = style.get('font-weight')
+        if fw in ('600', '700'):
+            style['font-weight'] = 'bold'
     for tag, style in iteritems(style_map):
         if style:
-            tag.set('style', '; '.join('{}: {}'.format(k, v) for k, v in iteritems(style)))
+            tag.set('style', '; '.join(f'{k}: {v}' for k, v in iteritems(style)))
         else:
             tag.attrib.pop('style', None)
     for span in root.xpath('//span[not(@style)]'):
@@ -247,7 +252,7 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
         f = QFontInfo(font)
         delta = tweaks['change_book_details_font_size_by'] + 1
         if delta:
-            font.setPixelSize(f.pixelSize() + delta)
+            font.setPixelSize(int(f.pixelSize() + delta))
             self.setFont(font)
         f = QFontMetrics(self.font())
         self.em_size = f.horizontalAdvance('m')
@@ -255,46 +260,51 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
         self._parent = weakref.ref(parent)
         self.comments_pat = re.compile(r'<!--.*?-->', re.DOTALL)
 
-        for rec in (
-            ('bold', 'format-text-bold', _('Bold'), True),
-            ('italic', 'format-text-italic', _('Italic'), True),
-            ('underline', 'format-text-underline', _('Underline'), True),
-            ('strikethrough', 'format-text-strikethrough', _('Strikethrough'), True),
-            ('superscript', 'format-text-superscript', _('Superscript'), True),
-            ('subscript', 'format-text-subscript', _('Subscript'), True),
-            ('ordered_list', 'format-list-ordered', _('Ordered list'), True),
-            ('unordered_list', 'format-list-unordered', _('Unordered list'), True),
-
-            ('align_left', 'format-justify-left', _('Align left'), True),
-            ('align_center', 'format-justify-center', _('Align center'), True),
-            ('align_right', 'format-justify-right', _('Align right'), True),
-            ('align_justified', 'format-justify-fill', _('Align justified'), True),
-            ('undo', 'edit-undo', _('Undo'), ),
-            ('redo', 'edit-redo', _('Redo'), ),
-            ('remove_format', 'edit-clear', _('Remove formatting'), ),
-            ('copy', 'edit-copy', _('Copy'), ),
-            ('paste', 'edit-paste', _('Paste'), ),
-            ('paste_and_match_style', 'edit-paste', _('Paste and match style'), ),
-            ('cut', 'edit-cut', _('Cut'), ),
-            ('indent', 'format-indent-more', _('Increase indentation'), ),
-            ('outdent', 'format-indent-less', _('Decrease indentation'), ),
-            ('select_all', 'edit-select-all', _('Select all'), ),
-
-            ('color', 'format-text-color', _('Foreground color')),
-            ('background', 'format-fill-color', _('Background color')),
-            ('insert_link', 'insert-link', _('Insert link or image'),),
-            ('insert_hr', 'format-text-hr', _('Insert separator'),),
-            ('clear', 'trash', _('Clear')),
-        ):
-            name, icon, text = rec[:3]
-            checkable = len(rec) == 4
-            ac = QAction(QIcon(I(icon + '.png')), text, self)
+        def r(name, icon, text, checkable=False, shortcut=None):
+            ac = QAction(QIcon.ic(icon + '.png'), text, self)
+            ac.setShortcutContext(Qt.ShortcutContext.WidgetWithChildrenShortcut)
             if checkable:
                 ac.setCheckable(checkable)
             setattr(self, 'action_'+name, ac)
             ac.triggered.connect(getattr(self, 'do_' + name))
+            if shortcut is not None:
+                sc = shortcut if isinstance(shortcut, QKeySequence) else QKeySequence(shortcut)
+                ac.setShortcut(sc)
+                ac.setToolTip(text + f' [{sc.toString(QKeySequence.SequenceFormat.NativeText)}]')
+            self.addAction(ac)
 
-        self.action_block_style = QAction(QIcon(I('format-text-heading.png')),
+        r('bold', 'format-text-bold', _('Bold'), True, QKeySequence.StandardKey.Bold)
+        r('italic', 'format-text-italic', _('Italic'), True, QKeySequence.StandardKey.Italic)
+        r('underline', 'format-text-underline', _('Underline'), True, QKeySequence.StandardKey.Underline)
+        r('strikethrough', 'format-text-strikethrough', _('Strikethrough'), True)
+        r('superscript', 'format-text-superscript', _('Superscript'), True)
+        r('subscript', 'format-text-subscript', _('Subscript'), True)
+        r('ordered_list', 'format-list-ordered', _('Ordered list'), True)
+        r('unordered_list', 'format-list-unordered', _('Unordered list'), True)
+
+        r('align_left', 'format-justify-left', _('Align left'), True)
+        r('align_center', 'format-justify-center', _('Align center'), True)
+        r('align_right', 'format-justify-right', _('Align right'), True)
+        r('align_justified', 'format-justify-fill', _('Align justified'), True)
+        r('undo', 'edit-undo', _('Undo'), shortcut=QKeySequence.StandardKey.Undo)
+        r('redo', 'edit-redo', _('Redo'), shortcut=QKeySequence.StandardKey.Redo)
+        r('remove_format', 'edit-clear', _('Remove formatting'))
+        r('copy', 'edit-copy', _('Copy'), shortcut=QKeySequence.StandardKey.Copy)
+        r('paste', 'edit-paste', _('Paste'), shortcut=QKeySequence.StandardKey.Paste)
+        r('paste_and_match_style', 'edit-paste', _('Paste and match style'))
+        r('cut', 'edit-cut', _('Cut'), shortcut=QKeySequence.StandardKey.Cut)
+        r('indent', 'format-indent-more', _('Increase indentation'))
+        r('outdent', 'format-indent-less', _('Decrease indentation'))
+        r('select_all', 'edit-select-all', _('Select all'), shortcut=QKeySequence.StandardKey.SelectAll)
+
+        r('color', 'format-text-color', _('Foreground color'))
+        r('background', 'format-fill-color', _('Background color'))
+        r('insert_link', 'insert-link', _('Insert link or image'),
+          shortcut=QKeySequence('Ctrl+l', QKeySequence.SequenceFormat.PortableText))
+        r('insert_hr', 'format-text-hr', _('Insert separator'),)
+        r('clear', 'trash', _('Clear'))
+
+        self.action_block_style = QAction(QIcon.ic('format-text-heading.png'),
                 _('Style text block'), self)
         self.action_block_style.setToolTip(
                 _('Style the selected text block'))
@@ -332,21 +342,6 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
         self.textChanged.connect(self.data_changed)
         self.update_cursor_position_actions()
 
-    def keyPressEvent(self, ev):
-        if ev.matches(QKeySequence.StandardKey.Bold):
-            ev.accept()
-            self.action_bold.toggle(), self.action_bold.trigger()
-            return
-        if ev.matches(QKeySequence.StandardKey.Italic):
-            ev.accept()
-            self.action_italic.toggle(), self.action_italic.trigger()
-            return
-        if ev.matches(QKeySequence.StandardKey.Underline):
-            ev.accept()
-            self.action_underline.toggle(), self.action_underline.trigger()
-            return
-        return QTextEdit.keyPressEvent(self, ev)
-
     def update_clipboard_actions(self, copy_available):
         self.action_copy.setEnabled(copy_available)
         self.action_cut.setEnabled(copy_available)
@@ -356,10 +351,10 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
 
     def update_cursor_position_actions(self):
         c = self.textCursor()
+        tcf = c.charFormat()
         ls = c.currentList()
         self.action_ordered_list.setChecked(ls is not None and ls.format().style() == QTextListFormat.Style.ListDecimal)
         self.action_unordered_list.setChecked(ls is not None and ls.format().style() == QTextListFormat.Style.ListDisc)
-        tcf = c.charFormat()
         vert = tcf.verticalAlignment()
         self.action_superscript.setChecked(vert == QTextCharFormat.VerticalAlignment.AlignSuperScript)
         self.action_subscript.setChecked(vert == QTextCharFormat.VerticalAlignment.AlignSubScript)
@@ -379,7 +374,7 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
             if bf.leftMargin() == bf.rightMargin() and bf.leftMargin() > 0:
                 name = 'blockquote'
         else:
-            name = 'h{}'.format(lvl)
+            name = f'h{lvl}'
         for ac in self.block_style_actions:
             ac.setChecked(ac.block_name == name)
 
@@ -405,30 +400,35 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
             fmt.setFontWeight(
                 QFont.Weight.Bold if c.charFormat().fontWeight() != QFont.Weight.Bold else QFont.Weight.Normal)
             c.mergeCharFormat(fmt)
+        self.update_cursor_position_actions()
 
     def do_italic(self):
         with self.editing_cursor() as c:
             fmt = QTextCharFormat()
             fmt.setFontItalic(not c.charFormat().fontItalic())
             c.mergeCharFormat(fmt)
+        self.update_cursor_position_actions()
 
     def do_underline(self):
         with self.editing_cursor() as c:
             fmt = QTextCharFormat()
             fmt.setFontUnderline(not c.charFormat().fontUnderline())
             c.mergeCharFormat(fmt)
+        self.update_cursor_position_actions()
 
     def do_strikethrough(self):
         with self.editing_cursor() as c:
             fmt = QTextCharFormat()
             fmt.setFontStrikeOut(not c.charFormat().fontStrikeOut())
             c.mergeCharFormat(fmt)
+        self.update_cursor_position_actions()
 
     def do_vertical_align(self, which):
         with self.editing_cursor() as c:
             fmt = QTextCharFormat()
             fmt.setVerticalAlignment(which)
             c.mergeCharFormat(fmt)
+        self.update_cursor_position_actions()
 
     def do_superscript(self):
         self.do_vertical_align(QTextCharFormat.VerticalAlignment.AlignSuperScript)
@@ -448,6 +448,7 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
                     ls.setFormat(lf)
             else:
                 ls = c.createList(fmt)
+        self.update_cursor_position_actions()
 
     def do_ordered_list(self):
         self.do_list(QTextListFormat.Style.ListDecimal)
@@ -457,9 +458,11 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
 
     def do_alignment(self, which):
         with self.editing_cursor() as c:
+            c = self.textCursor()
             fmt = QTextBlockFormat()
             fmt.setAlignment(which)
-            c.setBlockFormat(fmt)
+            c.mergeBlockFormat(fmt)
+        self.update_cursor_position_actions()
 
     def do_align_left(self):
         self.do_alignment(Qt.AlignmentFlag.AlignLeft)
@@ -485,6 +488,7 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
         with self.editing_cursor() as c:
             c.setBlockFormat(QTextBlockFormat())
             c.setCharFormat(QTextCharFormat())
+        self.update_cursor_position_actions()
 
     def do_copy(self):
         self.copy()
@@ -508,6 +512,7 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
             bf = c.blockFormat()
             bf.setTextIndent(bf.textIndent() + 2 * self.em_size * mult)
             c.setBlockFormat(bf)
+        self.update_cursor_position_actions()
 
     def do_indent(self):
         self.indent_block()
@@ -568,6 +573,7 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
             c.mergeBlockFormat(bf)
             if pos is not None:
                 c.setPosition(pos)
+        self.update_cursor_position_actions()
 
     def do_color(self):
         col = QColorDialog.getColor(Qt.GlobalColor.black, self,
@@ -598,7 +604,7 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
             return
         url = self.parse_link(link)
         if url.isValid():
-            url = unicode_type(url.toString(NO_URL_FORMATTING))
+            url = str(url.toString(NO_URL_FORMATTING))
             self.focus_self()
             with self.editing_cursor() as c:
                 if is_image:
@@ -651,7 +657,7 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
         d.setMinimumWidth(600)
         d.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok|QDialogButtonBox.StandardButton.Cancel)
         d.br = b = QPushButton(_('&Browse'))
-        b.setIcon(QIcon(I('document_open.png')))
+        b.setIcon(QIcon.ic('document_open.png'))
 
         def cf():
             filetypes = []
@@ -687,8 +693,8 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
         d.bb.rejected.connect(d.reject)
         d.resize(d.sizeHint())
         link, name, is_image = None, None, False
-        if d.exec_() == QDialog.DialogCode.Accepted:
-            link, name = unicode_type(d.url.text()).strip(), unicode_type(d.name.text()).strip()
+        if d.exec() == QDialog.DialogCode.Accepted:
+            link, name = str(d.url.text()).strip(), str(d.name.text()).strip()
             is_image = d.treat_as_image.isChecked()
         return link, name, is_image
 
@@ -727,7 +733,10 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
         if not check and '<img' not in raw.lower():
             return ''
 
-        root = parse(raw, maybe_xhtml=False, sanitize_names=True)
+        try:
+            root = parse(raw, maybe_xhtml=False, sanitize_names=True)
+        except Exception:
+            root = parse(clean_xml_chars(raw), maybe_xhtml=False, sanitize_names=True)
         if root.xpath('//meta[@name="calibre-dont-sanitize"]'):
             # Bypass cleanup if special meta tag exists
             return original_html
@@ -745,7 +754,7 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
                 x.tag not in ('script', 'style')]
 
         if len(elems) > 1:
-            ans = '<div>%s</div>'%(u''.join(elems))
+            ans = '<div>%s</div>'%(''.join(elems))
         else:
             ans = ''.join(elems)
             if not ans.startswith('<'):
@@ -769,7 +778,7 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
                 try:
                     with lopen(path, 'rb') as f:
                         data = f.read()
-                except EnvironmentError:
+                except OSError:
                     if path.rpartition('.')[-1].lower() in {'jpg', 'jpeg', 'gif', 'png', 'bmp', 'webp'}:
                         return QByteArray(bytearray.fromhex(
                                     '89504e470d0a1a0a0000000d49484452'
@@ -792,10 +801,16 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
 
     def text(self):
         return self.textCursor().selectedText()
+    selectedText = text
 
     def setText(self, text):
         with self.editing_cursor() as c:
             c.insertText(text)
+    insert = setText
+
+    def hasSelectedText(self):
+        c = self.textCursor()
+        return c.hasSelection()
 
     def contextMenuEvent(self, ev):
         menu = self.createStandardContextMenu()
@@ -825,7 +840,7 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
         am.addAction(self.action_background)
         am.addAction(self.action_color)
         menu.addAction(_('Smarten punctuation'), parent.smarten_punctuation)
-        menu.exec_(ev.globalPos())
+        menu.exec(ev.globalPos())
 
 # }}}
 
@@ -1062,12 +1077,7 @@ class Editor(QWidget):  # {{{
     def __init__(self, parent=None, one_line_toolbar=False, toolbar_prefs_name=None):
         QWidget.__init__(self, parent)
         self.toolbar_prefs_name = toolbar_prefs_name or self.toolbar_prefs_name
-        self.toolbar1 = QToolBar(self)
-        self.toolbar2 = QToolBar(self)
-        self.toolbar3 = QToolBar(self)
-        for i in range(1, 4):
-            t = getattr(self, 'toolbar%d'%i)
-            t.setIconSize(QSize(18, 18))
+        self.toolbar = create_flow_toolbar(self, restrict_to_single_line=one_line_toolbar, icon_size=18)
         self.editor = EditorWidget(self)
         self.editor.data_changed.connect(self.data_changed)
         self.set_base_url = self.editor.set_base_url
@@ -1076,6 +1086,7 @@ class Editor(QWidget):  # {{{
         self.tabs.setTabPosition(QTabWidget.TabPosition.South)
         self.wyswyg = QWidget(self.tabs)
         self.code_edit = QPlainTextEdit(self.tabs)
+        self.code_edit.setTabChangesFocus(True)
         self.source_dirty = False
         self.wyswyg_dirty = True
 
@@ -1083,14 +1094,8 @@ class Editor(QWidget):  # {{{
         self.wyswyg.layout = l = QVBoxLayout(self.wyswyg)
         self.setLayout(self._layout)
         l.setContentsMargins(0, 0, 0, 0)
-        if one_line_toolbar:
-            tb = QHBoxLayout()
-            l.addLayout(tb)
-        else:
-            tb = l
-        tb.addWidget(self.toolbar1)
-        tb.addWidget(self.toolbar2)
-        tb.addWidget(self.toolbar3)
+
+        l.addWidget(self.toolbar)
         l.addWidget(self.editor)
         self._layout.addWidget(self.tabs)
         self.tabs.addTab(self.wyswyg, _('&Normal view'))
@@ -1103,53 +1108,47 @@ class Editor(QWidget):  # {{{
             if hidden:
                 self.hide_toolbars()
 
-        # toolbar1 {{{
-        self.toolbar1.addAction(self.editor.action_undo)
-        self.toolbar1.addAction(self.editor.action_redo)
-        self.toolbar1.addAction(self.editor.action_select_all)
-        self.toolbar1.addAction(self.editor.action_remove_format)
-        self.toolbar1.addAction(self.editor.action_clear)
-        self.toolbar1.addSeparator()
+        self.toolbar.add_action(self.editor.action_undo)
+        self.toolbar.add_action(self.editor.action_redo)
+        self.toolbar.add_action(self.editor.action_select_all)
+        self.toolbar.add_action(self.editor.action_remove_format)
+        self.toolbar.add_action(self.editor.action_clear)
+        self.toolbar.add_separator()
 
         for x in ('copy', 'cut', 'paste'):
             ac = getattr(self.editor, 'action_'+x)
-            self.toolbar1.addAction(ac)
+            self.toolbar.add_action(ac)
 
-        self.toolbar1.addSeparator()
-        self.toolbar1.addAction(self.editor.action_background)
-        # }}}
+        self.toolbar.add_separator()
+        self.toolbar.add_action(self.editor.action_background)
+        self.toolbar.add_action(self.editor.action_color)
+        self.toolbar.add_separator()
 
-        # toolbar2 {{{
         for x in ('', 'un'):
             ac = getattr(self.editor, 'action_%sordered_list'%x)
-            self.toolbar2.addAction(ac)
-        self.toolbar2.addSeparator()
+            self.toolbar.add_action(ac)
+        self.toolbar.add_separator()
         for x in ('superscript', 'subscript', 'indent', 'outdent'):
-            self.toolbar2.addAction(getattr(self.editor, 'action_' + x))
+            self.toolbar.add_action(getattr(self.editor, 'action_' + x))
             if x in ('subscript', 'outdent'):
-                self.toolbar2.addSeparator()
+                self.toolbar.add_separator()
 
-        self.toolbar2.addAction(self.editor.action_block_style)
-        w = self.toolbar2.widgetForAction(self.editor.action_block_style)
-        if hasattr(w, 'setPopupMode'):
-            w.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
-        self.toolbar2.addAction(self.editor.action_insert_link)
-        self.toolbar2.addAction(self.editor.action_insert_hr)
-        # }}}
+        self.toolbar.add_action(self.editor.action_block_style, popup_mode=QToolButton.ToolButtonPopupMode.InstantPopup)
+        self.toolbar.add_action(self.editor.action_insert_link)
+        self.toolbar.add_action(self.editor.action_insert_hr)
+        self.toolbar.add_separator()
 
-        # toolbar3 {{{
         for x in ('bold', 'italic', 'underline', 'strikethrough'):
             ac = getattr(self.editor, 'action_'+x)
-            self.toolbar3.addAction(ac)
+            self.toolbar.add_action(ac)
             self.addAction(ac)
-        self.toolbar3.addSeparator()
+        self.toolbar.add_separator()
 
         for x in ('left', 'center', 'right', 'justified'):
             ac = getattr(self.editor, 'action_align_'+x)
-            self.toolbar3.addAction(ac)
-        self.toolbar3.addSeparator()
-        self.toolbar3.addAction(self.editor.action_color)
-        # }}}
+            self.toolbar.add_action(ac)
+        self.toolbar.add_separator()
+        QTimer.singleShot(0, self.toolbar.updateGeometry)
 
         self.code_edit.textChanged.connect(self.code_dirtied)
         self.editor.data_changed.connect(self.wyswyg_dirtied)
@@ -1193,14 +1192,11 @@ class Editor(QWidget):  # {{{
         self.source_dirty = True
 
     def hide_toolbars(self):
-        self.toolbar1.setVisible(False)
-        self.toolbar2.setVisible(False)
-        self.toolbar3.setVisible(False)
+        self.toolbar.setVisible(False)
 
     def show_toolbars(self):
-        self.toolbar1.setVisible(True)
-        self.toolbar2.setVisible(True)
-        self.toolbar3.setVisible(True)
+        self.toolbar.setVisible(True)
+        QTimer.singleShot(0, self.toolbar.updateGeometry)
 
     def toggle_toolbars(self):
         visible = self.toolbars_visible
@@ -1210,7 +1206,7 @@ class Editor(QWidget):  # {{{
 
     @property
     def toolbars_visible(self):
-        return self.toolbar1.isVisible() or self.toolbar2.isVisible() or self.toolbar3.isVisible()
+        return self.toolbar.isVisible()
 
     @toolbars_visible.setter
     def toolbars_visible(self, val):
@@ -1235,12 +1231,13 @@ class Editor(QWidget):  # {{{
 if __name__ == '__main__':
     from calibre.gui2 import Application
     app = Application([])
-    w = Editor()
+    w = Editor(one_line_toolbar=False)
     w.resize(800, 600)
+    w.setWindowFlag(Qt.WindowType.Dialog)
     w.show()
     w.html = '''<h1>Test Heading</h1><blockquote>Test blockquote</blockquote><p><span style="background-color: rgb(0, 255, 255); ">He hadn't
     set <u>out</u> to have an <em>affair</em>, <span style="font-style:italic; background-color:red">
     much</span> less a <s>long-term</s>, <b>devoted</b> one.</span><p>hello'''
     w.html = '<div><p id="moo">Testing <em>a</em> link.</p><p>\xa0</p><p>ss</p></div>'
-    app.exec_()
+    app.exec()
     # print w.html
